"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es.promise.js");

require("core-js/modules/es.regexp.flags.js");

require("core-js/modules/web.url.to-json.js");

var _ss3Accessory = _interopRequireDefault(require("./ss3Accessory"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const fahrenheitToCelsius = f => (f - 32.0) * 5.0 / 9.0;

class SS3FreezeSensor extends _ss3Accessory.default {
  constructor(name, id, log, debug, simplisafe, api) {
    super(name, id, log, debug, simplisafe, api);
    this.reachable = true;
    this.services.push(this.api.hap.Service.TemperatureSensor);
    this.startListening();
  }

  setAccessory(accessory) {
    super.setAccessory(accessory);
    this.accessory.getService(this.api.hap.Service.AccessoryInformation).setCharacteristic(this.api.hap.Characteristic.Manufacturer, 'SimpliSafe').setCharacteristic(this.api.hap.Characteristic.Model, 'Freeze Sensor').setCharacteristic(this.api.hap.Characteristic.SerialNumber, this.id);
    this.service = this.accessory.getService(this.api.hap.Service.TemperatureSensor);
    this.service.getCharacteristic(this.api.hap.Characteristic.CurrentTemperature).on('get', async callback => this.getState(callback));
    this.service.getCharacteristic(this.api.hap.Characteristic.StatusLowBattery).on('get', async callback => this.getBatteryStatus(callback));
    this.refreshState();
  }

  async updateReachability() {
    try {
      let sensors = await this.simplisafe.getSensors();
      let sensor = sensors.find(sen => sen.serial === this.id);

      if (!sensor) {
        this.reachable = false;
      } else {
        if (sensor.flags) {
          this.reachable = !sensor.flags.offline;
        } else {
          this.reachable = false;
        }
      }

      return this.reachable;
    } catch (err) {
      this.log.error(`An error occurred while updating reachability for ${this.name}`);
      this.log.error(err);
    }
  }

  async getSensorInformation() {
    try {
      let sensors = await this.simplisafe.getSensors(true);
      let sensor = sensors.find(sen => sen.serial === this.id);

      if (!sensor) {
        throw new Error('Could not find sensor');
      }

      return sensor;
    } catch (err) {
      throw new Error('An error occurred while getting sensor:', err.toJSON ? err.toJSON() : err);
    }
  }

  async getState(callback, forceRefresh = false) {
    if (this.simplisafe.isBlocked && Date.now() < this.simplisafe.nextAttempt) {
      return callback(new Error('Request blocked (rate limited)'));
    }

    if (!forceRefresh) {
      let characteristic = this.service.getCharacteristic(this.api.hap.Characteristic.CurrentTemperature);
      return callback(null, characteristic.value);
    }

    try {
      let sensor = await this.getSensorInformation();

      if (!sensor.status) {
        throw new Error('Sensor response not understood');
      }

      let temperature = fahrenheitToCelsius(sensor.status.temperature);
      callback(null, temperature);
    } catch (err) {
      callback(new Error(`An error occurred while getting sensor state: ${err}`));
    }
  }

  async getBatteryStatus(callback) {
    // No need to ping API for this and HomeKit is not very patient when waiting for it
    let characteristic = this.service.getCharacteristic(this.api.hap.Characteristic.StatusLowBattery);
    return callback(null, characteristic.value);
  }

  startListening() {
    this.simplisafe.subscribeToSensor(this.id, sensor => {
      if (this.service) {
        if (sensor.status) {
          let temperature = fahrenheitToCelsius(sensor.status.temperature);
          this.service.updateCharacteristic(this.api.hap.Characteristic.CurrentTemperature, temperature);
        }

        if (sensor.flags) {
          if (sensor.flags.lowBattery) {
            this.service.updateCharacteristic(this.api.hap.Characteristic.StatusLowBattery, this.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
          } else {
            this.service.updateCharacteristic(this.api.hap.Characteristic.StatusLowBattery, this.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
          }
        }
      }
    });
  }

  async refreshState() {
    if (this.debug) this.log('Refreshing sensor state');

    try {
      let sensor = await this.getSensorInformation();

      if (!sensor.status || !sensor.flags) {
        throw new Error('Sensor response not understood');
      }

      let temperature = fahrenheitToCelsius(sensor.status.temperature);
      let batteryLow = sensor.flags.lowBattery;
      let homekitBatteryState = batteryLow ? this.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW : this.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL;
      this.service.updateCharacteristic(this.api.hap.Characteristic.CurrentTemperature, temperature);
      this.service.updateCharacteristic(this.api.hap.Characteristic.StatusLowBattery, homekitBatteryState);
      if (this.debug) this.log(`Updated current temperature, battery state for ${this.name}: ${temperature}, ${batteryLow}`);
    } catch (err) {
      this.log.error('An error occurred while refreshing state');
      this.log.error(err);
    }
  }

}

var _default = SS3FreezeSensor;
exports.default = _default;